﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Threading;
using System.IO; // For DirectoryInfo
using System.Diagnostics; // For process
using Microsoft.Win32;// Registry
using System.ServiceProcess;

namespace AccoreConsoleDemo
{
    /// <summary>
    /// A set of utility methods
    /// </summary>
    class Utilities
    {
        /// <summary>
        /// Launches the accoreconsole.exe by specifying the parameter as shown here :
        /// The usage of accoreconsole.exe is :
        /// accoreconsole.exe /i <DrawingFilePath> /s <ScriptFilePath>
        /// </summary>
        /// <param name="ApplicationPath">Full Path to accoreconsole.ex</param>
        /// <param name="drawingFilePath">Full path to the drawing file</param>
        /// <param name="scriptFilePath">Full path to the script file that need to be executed</param>
        /// <param name="maxWaitInMilliSeconds">Max duration in ms before we give up waiting for accoreconsole.exe to finish</param>
        /// <returns></returns>
        internal static Thread RunAccoreconsoleApp(string ApplicationPath, string drawingFilePath, string scriptFilePath, int maxWaitInMilliSeconds)
        {
            Thread workerthread = new Thread(delegate()
            {
                if (File.Exists(ApplicationPath) == false)
                {
                    LogOutput("Location of accoreconsole.exe not found !");
                }
                else
                {
                    // Kill the accoreconsole.exe if it is already running...
                    Process[] processes = Process.GetProcessesByName(Path.GetFileNameWithoutExtension(ApplicationPath));
                    foreach (Process proc in processes)
                    {
                        proc.Kill();
                    }

                    // Prepare the arguments
                    string ApplicationArguments = string.Format("/i \"{0}\" /s \"{1}\" /l en-US", drawingFilePath, scriptFilePath);

                    Process ProcessObj = new Process();
                    ProcessObj.StartInfo.FileName = ApplicationPath;
                    ProcessObj.StartInfo.Arguments = ApplicationArguments;

                    if (Properties.Settings.Default.HideConsoleWindow == false)
                    {// Display the accoreconsole window
                        ProcessObj.StartInfo.UseShellExecute = true;
                        ProcessObj.StartInfo.CreateNoWindow = false;
                    }
                    else
                    {// Do not display the accoreconsole window
                        ProcessObj.StartInfo.UseShellExecute = false;
                        ProcessObj.StartInfo.CreateNoWindow = true;
                        ProcessObj.StartInfo.RedirectStandardOutput = true;
                    }

                    // Launch
                    ProcessObj.Start();

                    // Wait for the exe to complete.
                    ProcessObj.WaitForExit(maxWaitInMilliSeconds);

                    // Get the console output of accoreconsole.exe and log it to a file.
                    if (Properties.Settings.Default.HideConsoleWindow)
                    {
                        LogOutput(ProcessObj.StandardOutput.ReadToEnd());
                    }
                    else
                    {
                        LogOutput("Console output not available if \"Hide Console Window\" setting is unchecked.");
                    }
                }
            });

            workerthread.Start();
            return workerthread;
        }

        /// <summary>
        /// Creates a text file (.scr) with the specified contents
        /// </summary>
        /// <param name="filePath"></param>
        /// <param name="sContent"></param>
        internal static void CreateScriptFile(String filePath, String sContent)
        {
            string dirPath = Path.GetDirectoryName(filePath);
            if (Directory.Exists(dirPath) == false)
                Directory.CreateDirectory(dirPath);

            DeleteIfExists(filePath);

            TextWriter tw = new StreamWriter(filePath, false, Encoding.Unicode);
            tw.Write(sContent);
            tw.Flush();
            tw.Close();
        }

        /// <summary>
        /// Erase a file if it already exists
        /// </summary>
        /// <param name="filePath"></param>
        /// <returns></returns>
        internal static bool DeleteIfExists(String filePath)
        {
            try
            {
                if (File.Exists(filePath))
                    File.Delete(filePath);
            }
            catch
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Gets the ACAD 2013 install path from the registry.
        /// </summary>
        /// <returns></returns>
        internal static string GetAcad2013InstallPath()
        {
            String s_CLSID = String.Empty;
            String s_AcadVer = String.Empty;
            String s_AcadInstallPath = String.Empty;
            String s_ReleaseNumber = String.Empty;

            // ACAD 2013
            s_AcadVer = @"R19.0\";
            s_ReleaseNumber = "ACAD-B001:409";
            s_AcadInstallPath = GetRegistryValueHKLM(@"Software\Autodesk\AutoCAD\" + s_AcadVer + s_ReleaseNumber, "AcadLocation");

            return s_AcadInstallPath;
        }

        /// <summary>
        /// Helper method to read the registry
        /// </summary>
        /// <param name="subKey"></param>
        /// <param name="keyName"></param>
        /// <returns></returns>
        internal static string GetRegistryValueHKLM(string subKey, string keyName)
        {
            string value = string.Empty;
            RegistryKey localMac = Registry.LocalMachine;
            RegistryKey registrySubKey = localMac.OpenSubKey(subKey);

            if (registrySubKey != null)
            {
                object regKey = registrySubKey.GetValue(keyName);
                if (regKey != null)
                {
                    value = regKey.ToString();
                }
            }
            return value;
        }

        /// <summary>
        /// dumps the contents to a log file.
        /// </summary>
        /// <param name="sContent"></param>
        internal static void LogOutput(string sContent)
        {
            string logFilePath = Path.Combine(AppPath, "LastRun.log");

            string dirPath = Path.GetDirectoryName(logFilePath);
            if (Directory.Exists(dirPath) == false)
                Directory.CreateDirectory(dirPath);

            DeleteIfExists(logFilePath);

            TextWriter tw = new StreamWriter(logFilePath);
            tw.Write(sContent);
            tw.Flush();
            tw.Close();
        }

        /// <summary>
        /// Retrieves the log file contents
        /// </summary>
        /// <returns></returns>
        internal static string GetLogText()
        {
            string sContent = string.Empty;
            string logFilePath = Path.Combine(Utilities.AppPath, "LastRun.log");
            if (File.Exists(logFilePath))
            {
                TextReader tr = new StreamReader(logFilePath);
                sContent = tr.ReadToEnd();
                sContent = sContent.Replace("\b", "");
                tr.Close();
            }
            return sContent;
        }

        /// <summary>
        /// Get the exe path of this demo. 
        /// Temporary file will be created with this as the base path.
        /// </summary>
        internal static string AppPath
        {
            get
            {
                return Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath);
            }
        }

        /// <summary>
        /// Opens the file using its default application.
        /// In case of a pdf file, the default app may be a PDF reader.
        /// In case of a dwf file, the default app could be a ADR.
        /// </summary>
        /// <param name="filePath"></param>
        internal static void OpenDefaultViewer(string filePath)
        {
            System.Diagnostics.ProcessStartInfo Print = new System.Diagnostics.ProcessStartInfo();
            Print.Verb = "open";
            Print.WindowStyle = ProcessWindowStyle.Maximized;
            Print.FileName = filePath;
            Print.UseShellExecute = true;
            try
            {
                System.Diagnostics.Process.Start(Print);
            }
            catch (Exception)
            {
                System.Windows.Forms.MessageBox.Show("Generated file : " + filePath);
            }
        }
    }
}
