﻿// (C) Copyright 2002-2011 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Autodesk.AutoCAD.Runtime; // split between AcCoreMgd.dll and AcMgd.dll
using Autodesk.AutoCAD.EditorInput; // split between AcCoreMgd.dll and AcMgd.dll
using Autodesk.AutoCAD.DatabaseServices; // From AcDbMgd.dll
using Autodesk.AutoCAD.ApplicationServices; // split between AcCoreMgd.dll and AcMgd.dll
using Autodesk.AutoCAD.Geometry;

[assembly: CommandClass(typeof(NetTest1.TestClass))]
namespace NetTest1
{
    public class TestClass
    {
        // Usage : EntSelect <First corner point> <Second corner point>
        // Select any drawing
        // EntSelect 0,0,0 30,30,0
        // (command "EntSelect" "0.0,0.0,0.0" "30.0,30.0,0.0")
        [CommandMethod("EntSelect")]
        public void EntSelectMethod()
        {
            DocumentCollection docs = Autodesk.AutoCAD.ApplicationServices.Core.Application.DocumentManager;
            Document activeDoc = docs.MdiActiveDocument;
            Editor ed = activeDoc.Editor;

            Point3d pt1 = Point3d.Origin;
            Point3d pt2 = Point3d.Origin;

            PromptPointOptions ppo1 = new PromptPointOptions("First corner point : ");
            PromptPointResult ppr1 = ed.GetPoint(ppo1);
            if (ppr1.Status == PromptStatus.OK)
            {
                pt1 = ppr1.Value;
            }
            else
            {
                ed.WriteMessage(string.Format("{0}Incorrect corner point.", Environment.NewLine));
                return;
            }

            PromptPointOptions ppo2 = new PromptPointOptions("Second corner point : ");
            PromptPointResult ppr2 = ed.GetPoint(ppo2);
            if (ppr2.Status == PromptStatus.OK)
            {
                pt2 = ppr2.Value;
            }
            else
            {
                ed.WriteMessage(string.Format("{0}Incorrect corner point.", Environment.NewLine));
                return;
            }

            // zoom extents for selection to work correctly.
            ModelZoomExtents();

            // Obtain the selection set of all the entities in the drawing.
            PromptSelectionResult psr1 = ed.SelectAll();
            if (psr1.Status == PromptStatus.OK)
            {
                PrintSelectionSet("SelectAll", psr1.Value);
            }

            // Obtain the selection set of entities for a specified window.
            PromptSelectionResult psr2 = ed.SelectWindow(pt1, pt2);
            if (psr2.Status == PromptStatus.OK)
            {
                PrintSelectionSet("SelectWindow", psr2.Value);
            }
            else
            {
                ed.WriteMessage(string.Format("{0}{1} : {2} entities", Environment.NewLine, "SelectWindow", 0));
            }
        }

        // Displays the number of entities and handles of the entities in the specified selection set
        private void PrintSelectionSet(string Title, SelectionSet ss)
        {
            DocumentCollection docs = Autodesk.AutoCAD.ApplicationServices.Core.Application.DocumentManager;
            Document activeDoc = docs.MdiActiveDocument;
            Editor ed = activeDoc.Editor;

            ed.WriteMessage(string.Format("{0}{1} : {2} entities", Environment.NewLine, Title, ss.Count));
            foreach (ObjectId oid in ss.GetObjectIds())
            {
                ed.WriteMessage(string.Format("{0}Handle : {1}", Environment.NewLine, oid.Handle.ToString()));
            }
        }

        public void SetViewportToExtents(Database db, ViewportTableRecord viewportTableRec)
        {
            //lets update the database extents first
            //true gives the best fit but will take time
            db.UpdateExt(true);

            //get the screen aspect ratio to calculate the height and width
            double scrRatio = (viewportTableRec.Width / viewportTableRec.Height);

            //prepare Matrix for DCS to WCS transformation
            Matrix3d matWCS2DCS = Matrix3d.PlaneToWorld(viewportTableRec.ViewDirection);

            //for DCS target point is the origin
            matWCS2DCS = Matrix3d.Displacement(viewportTableRec.Target - Point3d.Origin) * matWCS2DCS;

            //WCS Xaxis is twisted by twist angle
            matWCS2DCS = Matrix3d.Rotation(-viewportTableRec.ViewTwist,
                                            viewportTableRec.ViewDirection,
                                            viewportTableRec.Target)
                                            * matWCS2DCS;

            matWCS2DCS = matWCS2DCS.Inverse();

            //tranform the extents to the DCS defined by the viewdir
            Extents3d extents = new Extents3d(db.Extmin, db.Extmax);
            extents.TransformBy(matWCS2DCS);

            //width of the extents in current view
            double width = (extents.MaxPoint.X - extents.MinPoint.X);

            //height of the extents in current view
            double height = (extents.MaxPoint.Y - extents.MinPoint.Y);

            //get the view center point
            Point2d center = new Point2d((extents.MaxPoint.X + extents.MinPoint.X) * 0.5,
                                         (extents.MaxPoint.Y + extents.MinPoint.Y) * 0.5);

            //check if the width 'fits' in current window
            //if not then get the new height as per the viewports aspect ratio
            if (width > (height * scrRatio))
                height = width / scrRatio;

            viewportTableRec.Height = height;
            viewportTableRec.Width = height * scrRatio;
            viewportTableRec.CenterPoint = center;
        }

        /// <summary>
        /// Performs a zoom extents on the drawing
        /// </summary>
        [CommandMethod("ModelZoomExtents")]
        public void ModelZoomExtents()
        {
            DocumentCollection docs = Autodesk.AutoCAD.ApplicationServices.Core.Application.DocumentManager;
            Document doc = docs.MdiActiveDocument;

            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                ed.UpdateTiledViewportsInDatabase();

                ViewportTableRecord viewportTableRec = Tx.GetObject(ed.ActiveViewportId, OpenMode.ForWrite) as ViewportTableRecord;

                SetViewportToExtents(db, viewportTableRec);

                ed.UpdateTiledViewportsFromDatabase();

                Tx.Commit();
            }
        }
    }
}
